/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright (c) 2012 Alexander Block. All rights reserved.
 * Copyright (c) 2012 STRATO AG. All rights reserved.
 */

#ifndef	_SYS_FITS_H
#define	_SYS_FITS_H

#include <sys/inttypes.h>
#include <sys/types.h>
#include <sys/dmu.h>
#include <sys/vnode.h>

#ifdef	__cplusplus
extern "C" {
#endif

#define	FITS_SEND_STREAM_MAGIC "btrfs-stream"
#define	FITS_SEND_STREAM_VERSION 1

#define	FITS_SEND_BUF_SIZE 65536
#define	FITS_SEND_READ_SIZE 49152

typedef struct _fits_stream_header {
	char		fs_magic[sizeof (FITS_SEND_STREAM_MAGIC)];
	uint32_t	fs_version;
} __attribute__((__packed__)) fits_stream_header_t;

typedef struct _fits_cmd_header {
	/* len of the payload, not including header */
	uint32_t	fc_len;
	uint16_t	fc_cmd;
	/* the crc includes the header, but with fc_crc assumed as 0 */
	uint32_t	fc_crc;
} __attribute__((__packed__)) fits_cmd_header_t;

typedef struct _fits_attr_header {
	uint16_t	fa_type;
	/* len of the payload, not including header */
	uint16_t	fa_len;
} __attribute__((__packed__)) fits_attr_header_t;

/* commands */
#define	FITS_CMD_SUBVOL		1
#define	FITS_CMD_SNAPSHOT	2
#define	FITS_CMD_MKFILE		3
#define	FITS_CMD_MKDIR		4
#define	FITS_CMD_MKNOD		5
#define	FITS_CMD_MKFIFO		6
#define	FITS_CMD_MKSOCK		7
#define	FITS_CMD_SYMLINK	8
#define	FITS_CMD_RENAME		9
#define	FITS_CMD_LINK		10
#define	FITS_CMD_UNLINK		11
#define	FITS_CMD_RMDIR		12
#define	FITS_CMD_SET_XATTR	13
#define	FITS_CMD_REMOVE_XATTR	14
#define	FITS_CMD_WRITE		15
#define	FITS_CMD_CLONE		16
#define	FITS_CMD_TRUNCATE	17
#define	FITS_CMD_CHMOD		18
#define	FITS_CMD_CHOWN		19
#define	FITS_CMD_UTIMES		20
#define	FITS_CMD_END		21
#define	FITS_CMD_MAX		21

/* attributes */
#define	FITS_ATTR_UUID			1
#define	FITS_ATTR_CTRANSID		2
#define	FITS_ATTR_INO			3
#define	FITS_ATTR_SIZE			4
#define	FITS_ATTR_MODE			5
#define	FITS_ATTR_UID			6
#define	FITS_ATTR_GID			7
#define	FITS_ATTR_RDEV			8
#define	FITS_ATTR_CTIME			9
#define	FITS_ATTR_MTIME			10
#define	FITS_ATTR_ATIME			11
#define	FITS_ATTR_OTIME			12
#define	FITS_ATTR_XATTR_NAME		13
#define	FITS_ATTR_XATTR_DATA		14
#define	FITS_ATTR_PATH			15
#define	FITS_ATTR_PATH_TO		16
#define	FITS_ATTR_PATH_LINK		17
#define	FITS_ATTR_FILE_OFFSET		18
#define	FITS_ATTR_DATA			19
#define	FITS_ATTR_CLONE_UUID		20
#define	FITS_ATTR_CLONE_CTRANSID	21
#define	FITS_ATTR_CLONE_PATH		22
#define	FITS_ATTR_CLONE_OFFSET		23
#define	FITS_ATTR_CLONE_LEN		24
#define	FITS_ATTR_MAX			24

int fits_send(objset_t *tosnap, objset_t *fromsnap, int outfd, vnode_t *vp,
    offset_t *off);

#ifdef	__cplusplus
}
#endif

#endif	/* _SYS_FITS_H */
